@@ -73,41 +73,48 @@ asduReceivedHandler (void* parameter, int address, CS101_ASDU asdu)
             CS101_ASDU_getTypeID(asdu),
             CS101_ASDU_getNumberOfElements(asdu));
 
-    if (CS101_ASDU_getTypeID(asdu) == M_ME_TE_1) {
-
+    if (CS101_ASDU_getTypeID(asdu) == M_ME_TE_1)
+    {
         printf("  measured scaled values with CP56Time2a timestamp:\n");
 
         int i;
 
-        for (i = 0; i < CS101_ASDU_getNumberOfElements(asdu); i++) {
-
+        for (i = 0; i < CS101_ASDU_getNumberOfElements(asdu); i++)
+        {
             MeasuredValueScaledWithCP56Time2a io =
                     (MeasuredValueScaledWithCP56Time2a) CS101_ASDU_getElement(asdu, i);
 
-            printf("    IOA: %i value: %i\n",
-                    InformationObject_getObjectAddress((InformationObject) io),
-                    MeasuredValueScaled_getValue((MeasuredValueScaled) io)
-            );
+            if (io)
+            {
+                printf("    IOA: %i value: %i\n",
+                        InformationObject_getObjectAddress((InformationObject) io),
+                        MeasuredValueScaled_getValue((MeasuredValueScaled) io)
+                );
 
-            MeasuredValueScaledWithCP56Time2a_destroy(io);
+                MeasuredValueScaledWithCP56Time2a_destroy(io);
+            }
         }
     }
-    else if (CS101_ASDU_getTypeID(asdu) == M_SP_NA_1) {
+    else if (CS101_ASDU_getTypeID(asdu) == M_SP_NA_1)
+    {
         printf("  single point information:\n");
 
         int i;
 
-        for (i = 0; i < CS101_ASDU_getNumberOfElements(asdu); i++) {
-
+        for (i = 0; i < CS101_ASDU_getNumberOfElements(asdu); i++)
+        {
             SinglePointInformation io =
                     (SinglePointInformation) CS101_ASDU_getElement(asdu, i);
 
-            printf("    IOA: %i value: %i\n",
-                    InformationObject_getObjectAddress((InformationObject) io),
-                    SinglePointInformation_getValue((SinglePointInformation) io)
-            );
+            if (io)
+            {
+                printf("    IOA: %i value: %i\n",
+                        InformationObject_getObjectAddress((InformationObject) io),
+                        SinglePointInformation_getValue((SinglePointInformation) io)
+                );
 
-            SinglePointInformation_destroy(io);
+                SinglePointInformation_destroy(io);
+            }
         }
     }
     else if (CS101_ASDU_getTypeID(asdu) == C_TS_TA_1) {
