@@ -1,7 +1,7 @@
 /*
  *  time.c
  *
- *  Copyright 2013-2021 Michael Zillgith
+ *  Copyright 2013-2024 Michael Zillgith
  *
  *  This file is part of Platform Abstraction Layer (libpal)
  *  for libiec61850, libmms, and lib60870.
@@ -59,3 +59,14 @@ Hal_setTimeInNs(nsSinceEpoch nsTime)
     return SetSystemTime(&st);
 }
 
+msSinceEpoch
+Hal_getMonotonicTimeInMs()
+{
+    return (msSinceEpoch)GetTickCount64();
+}
+
+nsSinceEpoch
+Hal_getMonotonicTimeInNs()
+{
+    return (nsSinceEpoch)(GetTickCount64() * 1000000ULL);
+}
