@@ -27,30 +27,29 @@
 #include "cs104_connection.h"
 
 #include <limits.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdio.h>
 
 #include "cs104_frame.h"
-#include "hal_thread.h"
 #include "hal_socket.h"
-#include "tls_socket.h"
+#include "hal_thread.h"
 #include "hal_time.h"
 #include "lib_memory.h"
+#include "tls_socket.h"
 
 #include "apl_types_internal.h"
+#include "cs101_asdu_internal.h"
 #include "information_objects_internal.h"
 #include "lib60870_internal.h"
-#include "cs101_asdu_internal.h"
 
 struct sCS104_APCIParameters defaultAPCIParameters = {
-		/* .k = */ 12,
-		/* .w = */ 8,
-		/* .t0 = */ 10,
-		/* .t1 = */ 15,
-		/* .t2 = */ 10,
-		/* .t3 = */ 20
-};
+    /* .k = */ 12,
+    /* .w = */ 8,
+    /* .t0 = */ 10,
+    /* .t1 = */ 15,
+    /* .t2 = */ 10,
+    /* .t3 = */ 20};
 
 static struct sCS101_AppLayerParameters defaultAppLayerParameters = {
     /* .sizeOfTypeId =  */ 1,
@@ -59,14 +58,14 @@ static struct sCS101_AppLayerParameters defaultAppLayerParameters = {
     /* .originatorAddress = */ 0,
     /* .sizeOfCA = */ 2,
     /* .sizeOfIOA = */ 3,
-    /* .maxSizeOfASDU = */ 249
-};
+    /* .maxSizeOfASDU = */ 249};
 
 #ifndef HOST_NAME_MAX
 #define HOST_NAME_MAX 64
 #endif
 
-typedef enum {
+typedef enum
+{
     STATE_IDLE = 0,
     STATE_INACTIVE = 1,
     STATE_ACTIVE = 2,
@@ -74,12 +73,14 @@ typedef enum {
     STATE_WAITING_FOR_STOPDT_CON = 4
 } CS104_ConState;
 
-typedef struct {
+typedef struct
+{
     uint64_t sentTime; /* required for T1 timeout */
     int seqNo;
 } SentASDU;
 
-struct sCS104_Connection {
+struct sCS104_Connection
+{
     char hostname[HOST_NAME_MAX + 1];
     int tcpPort;
 
@@ -144,20 +145,19 @@ struct sCS104_Connection {
     void* rawMessageHandlerParameter;
 };
 
-
-static uint8_t STARTDT_ACT_MSG[] = { 0x68, 0x04, 0x07, 0x00, 0x00, 0x00 };
+static uint8_t STARTDT_ACT_MSG[] = {0x68, 0x04, 0x07, 0x00, 0x00, 0x00};
 #define STARTDT_ACT_MSG_SIZE 6
 
-static uint8_t TESTFR_ACT_MSG[] = { 0x68, 0x04, 0x43, 0x00, 0x00, 0x00 };
+static uint8_t TESTFR_ACT_MSG[] = {0x68, 0x04, 0x43, 0x00, 0x00, 0x00};
 #define TESTFR_ACT_MSG_SIZE 6
 
-static uint8_t TESTFR_CON_MSG[] = { 0x68, 0x04, 0x83, 0x00, 0x00, 0x00 };
+static uint8_t TESTFR_CON_MSG[] = {0x68, 0x04, 0x83, 0x00, 0x00, 0x00};
 #define TESTFR_CON_MSG_SIZE 6
 
-static uint8_t STOPDT_ACT_MSG[] = { 0x68, 0x04, 0x13, 0x00, 0x00, 0x00 };
+static uint8_t STOPDT_ACT_MSG[] = {0x68, 0x04, 0x13, 0x00, 0x00, 0x00};
 #define STOPDT_ACT_MSG_SIZE 6
 
-static uint8_t STARTDT_CON_MSG[] = { 0x68, 0x04, 0x0b, 0x00, 0x00, 0x00 };
+static uint8_t STARTDT_CON_MSG[] = {0x68, 0x04, 0x0b, 0x00, 0x00, 0x00};
 #define STARTDT_CON_MSG_SIZE 6
 
 static int
@@ -166,6 +166,9 @@ writeToSocket(CS104_Connection self, uint8_t* buf, int size)
     if (self->rawMessageHandler)
         self->rawMessageHandler(self->rawMessageHandlerParameter, buf, size, true);
 
+    if (self->socket == NULL || self->conState == STATE_IDLE)
+        return 0;
+
 #if (CONFIG_CS104_SUPPORT_TLS == 1)
     if (self->tlsSocket)
         return TLSSocket_write(self->tlsSocket, buf, size);
@@ -190,8 +193,8 @@ sendSMessage(CS104_Connection self)
 {
     uint8_t* msg = self->sMessage;
 
-    msg [4] = (uint8_t) ((self->receiveCount % 128) * 2);
-    msg [5] = (uint8_t) (self->receiveCount / 128);
+    msg[4] = (uint8_t)((self->receiveCount % 128) * 2);
+    msg[5] = (uint8_t)(self->receiveCount / 128);
 
     writeToSocket(self, msg, 6);
 }
@@ -199,7 +202,7 @@ sendSMessage(CS104_Connection self)
 static int
 sendIMessage(CS104_Connection self, Frame frame)
 {
-    T104Frame_prepareToSend((T104Frame) frame, self->sendCount, self->receiveCount);
+    T104Frame_prepareToSend((T104Frame)frame, self->sendCount, self->receiveCount);
 
     writeToSocket(self, T104Frame_getBuffer(frame), T104Frame_getMsgSize(frame));
 
@@ -216,9 +219,10 @@ sendIMessage(CS104_Connection self, Frame frame)
 static CS104_Connection
 createConnection(const char* hostname, int tcpPort)
 {
-    CS104_Connection self = (CS104_Connection) GLOBAL_CALLOC(1, sizeof(struct sCS104_Connection));
+    CS104_Connection self = (CS104_Connection)GLOBAL_CALLOC(1, sizeof(struct sCS104_Connection));
 
-    if (self != NULL) {
+    if (self != NULL)
+    {
         strncpy(self->hostname, hostname, HOST_NAME_MAX);
         self->tcpPort = tcpPort;
         self->parameters = defaultAPCIParameters;
@@ -277,7 +281,8 @@ CS104_Connection_createSecure(const char* hostname, int tcpPort, TLSConfiguratio
 
     CS104_Connection self = createConnection(hostname, tcpPort);
 
-    if (self != NULL) {
+    if (self != NULL)
+    {
         self->tlsConfig = tlsConfig;
         TLSConfiguration_setClientMode(tlsConfig);
     }
@@ -316,9 +321,10 @@ resetConnection(CS104_Connection self)
     self->oldestSentASDU = -1;
     self->newestSentASDU = -1;
 
-    if (self->sentASDUs == NULL) {
+    if (self->sentASDUs == NULL)
+    {
         self->maxSentASDUs = self->parameters.k;
-        self->sentASDUs = (SentASDU*) GLOBAL_MALLOC(sizeof(SentASDU) * self->maxSentASDUs);
+        self->sentASDUs = (SentASDU*)GLOBAL_MALLOC(sizeof(SentASDU) * self->maxSentASDUs);
     }
 
     self->outstandingTestFCConMessages = 0;
@@ -342,18 +348,22 @@ checkSequenceNumber(CS104_Connection self, int seqNo)
     bool counterOverflowDetected = false;
     int oldestValidSeqNo = -1;
 
-    if (self->oldestSentASDU == -1) { /* if k-Buffer is empty */
+    if (self->oldestSentASDU == -1)
+    { /* if k-Buffer is empty */
         if (seqNo == self->sendCount)
             seqNoIsValid = true;
     }
-    else {
+    else
+    {
         /* Two cases are required to reflect sequence number overflow */
-        if (self->sentASDUs[self->oldestSentASDU].seqNo <= self->sentASDUs[self->newestSentASDU].seqNo) {
+        if (self->sentASDUs[self->oldestSentASDU].seqNo <= self->sentASDUs[self->newestSentASDU].seqNo)
+        {
             if ((seqNo >= self->sentASDUs[self->oldestSentASDU].seqNo) &&
                 (seqNo <= self->sentASDUs[self->newestSentASDU].seqNo))
                 seqNoIsValid = true;
         }
-        else {
+        else
+        {
             if ((seqNo >= self->sentASDUs[self->oldestSentASDU].seqNo) ||
                 (seqNo <= self->sentASDUs[self->newestSentASDU].seqNo))
                 seqNoIsValid = true;
@@ -371,20 +381,23 @@ checkSequenceNumber(CS104_Connection self, int seqNo)
             seqNoIsValid = true;
     }
 
-    if (seqNoIsValid) {
-
-        if (self->oldestSentASDU != -1) {
-
-            do {
-                if (counterOverflowDetected == false) {
-                    if (seqNo < self->sentASDUs [self->oldestSentASDU].seqNo)
+    if (seqNoIsValid)
+    {
+        if (self->oldestSentASDU != -1)
+        {
+            do
+            {
+                if (counterOverflowDetected == false)
+                {
+                    if (seqNo < self->sentASDUs[self->oldestSentASDU].seqNo)
                         break;
                 }
 
                 if (seqNo == oldestValidSeqNo)
                     break;
 
-                if (self->sentASDUs [self->oldestSentASDU].seqNo == seqNo) {
+                if (self->sentASDUs[self->oldestSentASDU].seqNo == seqNo)
+                {
                     /* we arrived at the seq# that has been confirmed */
 
                     if (self->oldestSentASDU == self->newestSentASDU)
@@ -399,13 +412,13 @@ checkSequenceNumber(CS104_Connection self, int seqNo)
 
                 int checkIndex = (self->newestSentASDU + 1) % self->maxSentASDUs;
 
-                if (self->oldestSentASDU == checkIndex) {
+                if (self->oldestSentASDU == checkIndex)
+                {
                     self->oldestSentASDU = -1;
                     break;
                 }
 
             } while (true);
-
         }
     }
 
@@ -460,7 +473,8 @@ CS104_Connection_destroy(CS104_Connection self)
     Semaphore_destroy(self->conStateLock);
 #endif
 
-    if (self->localIpAddress) {
+    if (self->localIpAddress)
+    {
         GLOBAL_FREEMEM(self->localIpAddress);
         self->localIpAddress = NULL;
     }
@@ -471,13 +485,16 @@ CS104_Connection_destroy(CS104_Connection self)
 void
 CS104_Connection_setLocalAddress(CS104_Connection self, const char* localIpAddress, int localPort)
 {
-    if (self) {
-        if (self->localIpAddress) {
+    if (self)
+    {
+        if (self->localIpAddress)
+        {
             GLOBAL_FREEMEM(self->localIpAddress);
             self->localIpAddress = NULL;
         }
 
-        if (localIpAddress) {
+        if (localIpAddress)
+        {
             self->localIpAddress = strdup(localIpAddress);
             self->localTcpPort = localPort;
         }
@@ -544,7 +561,8 @@ receiveMessage(CS104_Connection self)
     int bufPos = self->recvBufPos;
 
     /* read start byte */
-    if (bufPos == 0) {
+    if (bufPos == 0)
+    {
         int readFirst = readFromSocket(self, buffer, 1);
 
         if (readFirst < 1)
@@ -557,15 +575,18 @@ receiveMessage(CS104_Connection self)
     }
 
     /* read length byte */
-    if (bufPos == 1)  {
+    if (bufPos == 1)
+    {
 
         int readCnt = readFromSocket(self, buffer + 1, 1);
 
-        if (readCnt < 0) {
+        if (readCnt < 0)
+        {
             self->recvBufPos = 0;
             return -1;
         }
-        else if (readCnt == 0) {
+        else if (readCnt == 0)
+        {
             self->recvBufPos = 1;
             return 0;
         }
@@ -574,22 +595,26 @@ receiveMessage(CS104_Connection self)
     }
 
     /* read remaining frame */
-    if (bufPos > 1) {
+    if (bufPos > 1)
+    {
         int length = buffer[1];
 
         int remainingLength = length - bufPos + 2;
 
         int readCnt = readFromSocket(self, buffer + bufPos, remainingLength);
 
-        if (readCnt == remainingLength) {
+        if (readCnt == remainingLength)
+        {
             self->recvBufPos = 0;
             return length + 2;
         }
-        else if (readCnt == -1) {
+        else if (readCnt == -1)
+        {
             self->recvBufPos = 0;
             return -1;
         }
-        else {
+        else
+        {
             self->recvBufPos = bufPos + readCnt;
             return 0;
         }
@@ -602,8 +627,10 @@ receiveMessage(CS104_Connection self)
 static bool
 checkConfirmTimeout(CS104_Connection self, uint64_t currentTime)
 {
-    if (currentTime > self->lastConfirmationTime) {
-        if ((currentTime - self->lastConfirmationTime) >= (uint32_t) (self->parameters.t2 * 1000)) {
+    if (currentTime > self->lastConfirmationTime)
+    {
+        if ((currentTime - self->lastConfirmationTime) >= (uint32_t)(self->parameters.t2 * 1000))
+        {
             return true;
         }
     }
@@ -627,34 +654,38 @@ checkMessage(CS104_Connection self, uint8_t* buffer, int msgSize)
 
     if ((buffer[2] & 1) == 0) /* I format frame */
     {
-        if (self->timeoutT2Trigger == false) {
+        if (self->timeoutT2Trigger == false)
+        {
             self->timeoutT2Trigger = true;
             self->lastConfirmationTime = Hal_getMonotonicTimeInMs(); /* start timeout T2 */
         }
 
-        if (msgSize < 7) {
+        if (msgSize < 7)
+        {
             DEBUG_PRINT("I msg too small!\n");
-            retVal =  false;
+            retVal = false;
 
             goto exit_function;
         }
 
-        int frameSendSequenceNumber = ((buffer [3] * 0x100) + (buffer [2] & 0xfe)) / 2;
-        int frameRecvSequenceNumber = ((buffer [5] * 0x100) + (buffer [4] & 0xfe)) / 2;
+        int frameSendSequenceNumber = ((buffer[3] * 0x100) + (buffer[2] & 0xfe)) / 2;
+        int frameRecvSequenceNumber = ((buffer[5] * 0x100) + (buffer[4] & 0xfe)) / 2;
 
         DEBUG_PRINT("Received I frame: N(S) = %i N(R) = %i\n", frameSendSequenceNumber, frameRecvSequenceNumber);
 
         /* check the receive sequence number N(R) - connection will be closed on an unexpected value */
-        if (frameSendSequenceNumber != self->receiveCount) {
+        if (frameSendSequenceNumber != self->receiveCount)
+        {
             DEBUG_PRINT("Sequence error: Close connection!");
 
-            retVal =  false;
+            retVal = false;
 
             goto exit_function;
         }
 
-        if (checkSequenceNumber(self, frameRecvSequenceNumber) == false) {
-            retVal =  false;
+        if (checkSequenceNumber(self, frameRecvSequenceNumber) == false)
+        {
+            retVal = false;
 
             goto exit_function;
         }
@@ -664,7 +695,8 @@ checkMessage(CS104_Connection self, uint8_t* buffer, int msgSize)
 
         struct sCS101_ASDU _asdu;
 
-        CS101_ASDU asdu = CS101_ASDU_createFromBufferEx(&_asdu, (CS101_AppLayerParameters)&(self->alParameters), buffer + 6, msgSize - 6);
+        CS101_ASDU asdu = CS101_ASDU_createFromBufferEx(&_asdu, (CS101_AppLayerParameters) & (self->alParameters),
+                                                        buffer + 6, msgSize - 6);
 
         if (asdu)
         {
@@ -673,53 +705,58 @@ checkMessage(CS104_Connection self, uint8_t* buffer, int msgSize)
         }
         else
         {
-            retVal =  false;
+            retVal = false;
 
             goto exit_function;
         }
     }
-    else if ((buffer[2] & 0x03) == 0x03)  /* U format frame */
+    else if ((buffer[2] & 0x03) == 0x03) /* U format frame */
     {
         DEBUG_PRINT("Received U frame\n");
 
         self->uMessageTimeout = 0;
 
-        if (buffer[2] == 0x43) { /* Check for TESTFR_ACT message */
+        if (buffer[2] == 0x43)
+        { /* Check for TESTFR_ACT message */
             DEBUG_PRINT("Send TESTFR_CON\n");
 
             writeToSocket(self, TESTFR_CON_MSG, TESTFR_CON_MSG_SIZE);
         }
-        else if (buffer[2] == 0x83) { /* TESTFR_CON */
+        else if (buffer[2] == 0x83)
+        { /* TESTFR_CON */
             DEBUG_PRINT("Rcvd TESTFR_CON\n");
             self->outstandingTestFCConMessages = 0;
         }
-        else if (buffer[2] == 0x07) { /* STARTDT_ACT */
+        else if (buffer[2] == 0x07)
+        { /* STARTDT_ACT */
             DEBUG_PRINT("Send STARTDT_CON\n");
 
             writeToSocket(self, STARTDT_CON_MSG, STARTDT_CON_MSG_SIZE);
 
             self->conState = STATE_ACTIVE;
         }
-        else if (buffer[2] == 0x0b) { /* STARTDT_CON */
+        else if (buffer[2] == 0x0b)
+        { /* STARTDT_CON */
             DEBUG_PRINT("Received STARTDT_CON\n");
 
             self->conState = STATE_ACTIVE;
         }
-        else if (buffer[2] == 0x23) { /* STOPDT_CON */
+        else if (buffer[2] == 0x23)
+        { /* STOPDT_CON */
             DEBUG_PRINT("Received STOPDT_CON\n");
 
             self->conState = STATE_INACTIVE;
         }
-
     }
-    else if (buffer [2] == 0x01) /* S-message */
+    else if (buffer[2] == 0x01) /* S-message */
     {
         int seqNo = (buffer[4] + buffer[5] * 0x100) / 2;
 
         DEBUG_PRINT("Rcvd S(%i) (own sendcounter = %i)\n", seqNo, self->sendCount);
 
-        if (checkSequenceNumber(self, seqNo) == false) {
-            retVal =  false;
+        if (checkSequenceNumber(self, seqNo) == false)
+        {
+            retVal = false;
 
             goto exit_function;
         }
@@ -789,7 +826,8 @@ handleTimeouts(CS104_Connection self)
     {
         if (currentTime > self->sentASDUs[self->oldestSentASDU].sentTime)
         {
-            if ((currentTime - self->sentASDUs[self->oldestSentASDU].sentTime) >= (uint64_t) (self->parameters.t1 * 1000))
+            if ((currentTime - self->sentASDUs[self->oldestSentASDU].sentTime) >=
+                (uint64_t)(self->parameters.t1 * 1000))
             {
                 DEBUG_PRINT("I message timeout\n");
                 retVal = false;
@@ -864,7 +902,7 @@ isClose(CS104_Connection self)
 static void*
 handleConnection(void* parameter)
 {
-    CS104_Connection self = (CS104_Connection) parameter;
+    CS104_Connection self = (CS104_Connection)parameter;
 
     CS104_ConnectionEvent event = CS104_CONNECTION_OPENED;
 
@@ -872,21 +910,24 @@ handleConnection(void* parameter)
 
     self->socket = TcpSocket_create();
 
-    if (self->socket) {
+    if (self->socket)
+    {
         Socket_setConnectTimeout(self->socket, self->connectTimeoutInMs);
 
-        if (self->localIpAddress) {
+        if (self->localIpAddress)
+        {
             Socket_bind(self->socket, self->localIpAddress, self->localTcpPort);
         }
 
-        if (Socket_connect(self->socket, self->hostname, self->tcpPort)) {
-
+        if (Socket_connect(self->socket, self->hostname, self->tcpPort))
+        {
 #if (CONFIG_USE_SEMAPHORES == 1)
             Semaphore_wait(self->conStateLock);
 #endif /* (CONFIG_USE_SEMAPHORES == 1) */
 
 #if (CONFIG_CS104_SUPPORT_TLS == 1)
-            if (self->tlsConfig != NULL) {
+            if (self->tlsConfig != NULL)
+            {
                 self->tlsSocket = TLSSocket_create(self->socket, self->tlsConfig, false);
 
                 if (self->tlsSocket)
@@ -904,8 +945,8 @@ handleConnection(void* parameter)
             Semaphore_post(self->conStateLock);
 #endif /* (CONFIG_USE_SEMAPHORES == 1) */
 
-            if (isRunning(self)) {
-
+            if (isRunning(self))
+            {
 #if (CONFIG_USE_SEMAPHORES == 1)
                 Semaphore_wait(self->conStateLock);
 #endif /* (CONFIG_USE_SEMAPHORES == 1) */
@@ -924,15 +965,17 @@ handleConnection(void* parameter)
 
                 bool loopRunning = true;
 
-                while (loopRunning) {
-
+                while (loopRunning)
+                {
                     Handleset_reset(handleSet);
                     Handleset_addSocket(handleSet, self->socket);
 
-                    if (Handleset_waitReady(handleSet, 100)) {
+                    if (Handleset_waitReady(handleSet, 100))
+                    {
                         int bytesRec = receiveMessage(self);
 
-                        if (bytesRec == -1) {
+                        if (bytesRec == -1)
+                        {
                             loopRunning = false;
 
 #if (CONFIG_USE_SEMAPHORES == 1)
@@ -946,10 +989,11 @@ handleConnection(void* parameter)
 #endif /* (CONFIG_USE_SEMAPHORES == 1) */
                         }
 
-                        if (bytesRec > 0) {
-
+                        if (bytesRec > 0)
+                        {
                             if (self->rawMessageHandler)
-                                self->rawMessageHandler(self->rawMessageHandlerParameter, self->recvBuffer, bytesRec, false);
+                                self->rawMessageHandler(self->rawMessageHandlerParameter, self->recvBuffer, bytesRec,
+                                                        false);
 
 #if (CONFIG_USE_SEMAPHORES == 1)
                             Semaphore_wait(self->conStateLock);
@@ -975,9 +1019,11 @@ handleConnection(void* parameter)
                             if ((newState != oldState) && self->connectionHandler)
                             {
                                 if (newState == STATE_ACTIVE)
-                                    self->connectionHandler(self->connectionHandlerParameter, self, CS104_CONNECTION_STARTDT_CON_RECEIVED);
+                                    self->connectionHandler(self->connectionHandlerParameter, self,
+                                                            CS104_CONNECTION_STARTDT_CON_RECEIVED);
                                 else if (newState == STATE_INACTIVE)
-                                    self->connectionHandler(self->connectionHandlerParameter, self, CS104_CONNECTION_STOPDT_CON_RECEIVED);
+                                    self->connectionHandler(self->connectionHandlerParameter, self,
+                                                            CS104_CONNECTION_STOPDT_CON_RECEIVED);
                             }
                         }
 
@@ -985,7 +1031,9 @@ handleConnection(void* parameter)
                         Semaphore_wait(self->conStateLock);
 #endif /* (CONFIG_USE_SEMAPHORES == 1) */
 
-                        if ((self->unconfirmedReceivedIMessages >= self->parameters.w) || (self->conState == STATE_WAITING_FOR_STOPDT_CON)) {
+                        if ((self->unconfirmedReceivedIMessages >= self->parameters.w) ||
+                            (self->conState == STATE_WAITING_FOR_STOPDT_CON))
+                        {
                             confirmOutstandingMessages(self);
                         }
 
@@ -1007,7 +1055,8 @@ handleConnection(void* parameter)
                 event = CS104_CONNECTION_CLOSED;
             }
         }
-        else {
+        else
+        {
 #if (CONFIG_USE_SEMAPHORES == 1)
             Semaphore_wait(self->conStateLock);
 #endif /* (CONFIG_USE_SEMAPHORES == 1) */
@@ -1026,17 +1075,18 @@ handleConnection(void* parameter)
 #endif /* (CONFIG_USE_SEMAPHORES == 1) */
 
         /* Confirm all unconfirmed received I-messages before closing the connection */
-        if (self->unconfirmedReceivedIMessages > 0) {
+        if (self->unconfirmedReceivedIMessages > 0)
+        {
             confirmOutstandingMessages(self);
         }
 
-    #if (CONFIG_CS104_SUPPORT_TLS == 1)
+#if (CONFIG_CS104_SUPPORT_TLS == 1)
         if (self->tlsSocket)
         {
             TLSSocket_close(self->tlsSocket);
             self->tlsSocket = NULL;
         }
-    #endif
+#endif
 
         Socket_destroy(self->socket);
         self->socket = NULL;
@@ -1065,7 +1115,8 @@ handleConnection(void* parameter)
     }
 
     /* Call connection handler */
-    if ((event == CS104_CONNECTION_CLOSED) || (event == CS104_CONNECTION_FAILED)) {
+    if ((event == CS104_CONNECTION_CLOSED) || (event == CS104_CONNECTION_FAILED))
+    {
         if (self->connectionHandler)
             self->connectionHandler(self->connectionHandlerParameter, self, event);
     }
@@ -1090,12 +1141,13 @@ CS104_Connection_connectAsync(CS104_Connection self)
 #endif /* (CONFIG_USE_SEMAPHORES == 1) */
 
 #if (CONFIG_USE_THREADS == 1)
-    if (self->connectionHandlingThread) {
+    if (self->connectionHandlingThread)
+    {
         Thread_destroy(self->connectionHandlingThread);
         self->connectionHandlingThread = NULL;
     }
 
-    self->connectionHandlingThread = Thread_create(handleConnection, (void*) self, false);
+    self->connectionHandlingThread = Thread_create(handleConnection, (void*)self, false);
 
     if (self->connectionHandlingThread)
         Thread_start(self->connectionHandlingThread);
@@ -1135,33 +1187,33 @@ CS104_Connection_setRawMessageHandler(CS104_Connection self, IEC60870_RawMessage
 }
 
 static void
-encodeIdentificationField(CS104_Connection self, Frame frame, TypeID typeId,
-        int vsq, CS101_CauseOfTransmission cot, int ca)
+encodeIdentificationField(CS104_Connection self, Frame frame, TypeID typeId, int vsq, CS101_CauseOfTransmission cot,
+                          int ca)
 {
     T104Frame_setNextByte(frame, typeId);
-    T104Frame_setNextByte(frame, (uint8_t) vsq);
+    T104Frame_setNextByte(frame, (uint8_t)vsq);
 
     /* encode COT */
-    T104Frame_setNextByte(frame, (uint8_t) cot);
+    T104Frame_setNextByte(frame, (uint8_t)cot);
     if (self->alParameters.sizeOfCOT == 2)
-        T104Frame_setNextByte(frame, (uint8_t) self->alParameters.originatorAddress);
+        T104Frame_setNextByte(frame, (uint8_t)self->alParameters.originatorAddress);
 
     /* encode CA */
     T104Frame_setNextByte(frame, (uint8_t)(ca & 0xff));
     if (self->alParameters.sizeOfCA == 2)
-        T104Frame_setNextByte(frame, (uint8_t) ((ca & 0xff00) >> 8));
+        T104Frame_setNextByte(frame, (uint8_t)((ca & 0xff00) >> 8));
 }
 
 static void
 encodeIOA(CS104_Connection self, Frame frame, int ioa)
 {
-    T104Frame_setNextByte(frame, (uint8_t) (ioa & 0xff));
+    T104Frame_setNextByte(frame, (uint8_t)(ioa & 0xff));
 
     if (self->alParameters.sizeOfIOA > 1)
-        T104Frame_setNextByte(frame, (uint8_t) ((ioa / 0x100) & 0xff));
+        T104Frame_setNextByte(frame, (uint8_t)((ioa / 0x100) & 0xff));
 
     if (self->alParameters.sizeOfIOA > 2)
-        T104Frame_setNextByte(frame, (uint8_t) ((ioa / 0x10000) & 0xff));
+        T104Frame_setNextByte(frame, (uint8_t)((ioa / 0x10000) & 0xff));
 }
 
 void
@@ -1214,15 +1266,14 @@ sendIMessageAndUpdateSentASDUs(CS104_Connection self, Frame frame)
     {
         self->oldestSentASDU = 0;
         self->newestSentASDU = 0;
-
     }
     else
     {
         currentIndex = (self->newestSentASDU + 1) % self->maxSentASDUs;
     }
 
-    self->sentASDUs [currentIndex].seqNo = sendIMessage (self, frame);
-    self->sentASDUs [currentIndex].sentTime = Hal_getMonotonicTimeInMs();
+    self->sentASDUs[currentIndex].seqNo = sendIMessage(self, frame);
+    self->sentASDUs[currentIndex].sentTime = Hal_getMonotonicTimeInMs();
 
     self->newestSentASDU = currentIndex;
 }
@@ -1255,9 +1306,10 @@ sendASDUInternal(CS104_Connection self, Frame frame)
 }
 
 bool
-CS104_Connection_sendInterrogationCommand(CS104_Connection self, CS101_CauseOfTransmission cot, int ca, QualifierOfInterrogation qoi)
+CS104_Connection_sendInterrogationCommand(CS104_Connection self, CS101_CauseOfTransmission cot, int ca,
+                                          QualifierOfInterrogation qoi)
 {
-    Frame frame = (Frame) T104Frame_create();
+    Frame frame = (Frame)T104Frame_create();
 
     encodeIdentificationField(self, frame, C_IC_NA_1, 1, cot, ca);
 
@@ -1270,9 +1322,10 @@ CS104_Connection_sendInterrogationCommand(CS104_Connection self, CS101_CauseOfTr
 }
 
 bool
-CS104_Connection_sendCounterInterrogationCommand(CS104_Connection self, CS101_CauseOfTransmission cot, int ca, uint8_t qcc)
+CS104_Connection_sendCounterInterrogationCommand(CS104_Connection self, CS101_CauseOfTransmission cot, int ca,
+                                                 uint8_t qcc)
 {
-    Frame frame = (Frame) T104Frame_create();
+    Frame frame = (Frame)T104Frame_create();
 
     encodeIdentificationField(self, frame, C_CI_NA_1, 1, cot, ca);
 
@@ -1287,7 +1340,7 @@ CS104_Connection_sendCounterInterrogationCommand(CS104_Connection self, CS101_Ca
 bool
 CS104_Connection_sendReadCommand(CS104_Connection self, int ca, int ioa)
 {
-    Frame frame = (Frame) T104Frame_create();
+    Frame frame = (Frame)T104Frame_create();
 
     encodeIdentificationField(self, frame, C_RD_NA_1, 1, CS101_COT_REQUEST, ca);
 
@@ -1299,7 +1352,7 @@ CS104_Connection_sendReadCommand(CS104_Connection self, int ca, int ioa)
 bool
 CS104_Connection_sendClockSyncCommand(CS104_Connection self, int ca, CP56Time2a newTime)
 {
-    Frame frame = (Frame) T104Frame_create();
+    Frame frame = (Frame)T104Frame_create();
 
     encodeIdentificationField(self, frame, C_CS_NA_1, 1, CS101_COT_ACTIVATION, ca);
 
@@ -1313,7 +1366,7 @@ CS104_Connection_sendClockSyncCommand(CS104_Connection self, int ca, CP56Time2a
 bool
 CS104_Connection_sendTestCommand(CS104_Connection self, int ca)
 {
-    Frame frame = (Frame) T104Frame_create();
+    Frame frame = (Frame)T104Frame_create();
 
     encodeIdentificationField(self, frame, C_TS_NA_1, 1, CS101_COT_ACTIVATION, ca);
 
@@ -1332,34 +1385,36 @@ CS104_Connection_sendTestCommandWithTimestamp(CS104_Connection self, int ca, uin
 
     TestCommandWithCP56Time2a testCommand = TestCommandWithCP56Time2a_create(&_testCommand, tsc, timestamp);
 
-    return CS104_Connection_sendProcessCommandEx(self, CS101_COT_ACTIVATION, ca, (InformationObject) testCommand);
+    return CS104_Connection_sendProcessCommandEx(self, CS101_COT_ACTIVATION, ca, (InformationObject)testCommand);
 }
 
 bool
-CS104_Connection_sendProcessCommand(CS104_Connection self, TypeID typeId, CS101_CauseOfTransmission cot, int ca, InformationObject sc)
+CS104_Connection_sendProcessCommand(CS104_Connection self, TypeID typeId, CS101_CauseOfTransmission cot, int ca,
+                                    InformationObject sc)
 {
-    Frame frame = (Frame) T104Frame_create();
+    Frame frame = (Frame)T104Frame_create();
 
     if (typeId == 0)
         typeId = InformationObject_getType(sc);
 
-    encodeIdentificationField (self, frame, typeId, 1 /* SQ:false; NumIX:1 */, cot, ca);
+    encodeIdentificationField(self, frame, typeId, 1 /* SQ:false; NumIX:1 */, cot, ca);
 
-    InformationObject_encode(sc, frame, (CS101_AppLayerParameters) &(self->alParameters), false);
+    InformationObject_encode(sc, frame, (CS101_AppLayerParameters) & (self->alParameters), false);
 
     return sendASDUInternal(self, frame);
 }
 
 bool
-CS104_Connection_sendProcessCommandEx(CS104_Connection self, CS101_CauseOfTransmission cot, int ca, InformationObject sc)
+CS104_Connection_sendProcessCommandEx(CS104_Connection self, CS101_CauseOfTransmission cot, int ca,
+                                      InformationObject sc)
 {
-    Frame frame = (Frame) T104Frame_create();
+    Frame frame = (Frame)T104Frame_create();
 
     TypeID typeId = InformationObject_getType(sc);
 
-    encodeIdentificationField (self, frame, typeId, 1 /* SQ:false; NumIX:1 */, cot, ca);
+    encodeIdentificationField(self, frame, typeId, 1 /* SQ:false; NumIX:1 */, cot, ca);
 
-    InformationObject_encode(sc, frame, (CS101_AppLayerParameters) &(self->alParameters), false);
+    InformationObject_encode(sc, frame, (CS101_AppLayerParameters) & (self->alParameters), false);
 
     return sendASDUInternal(self, frame);
 }
@@ -1367,7 +1422,7 @@ CS104_Connection_sendProcessCommandEx(CS104_Connection self, CS101_CauseOfTransm
 bool
 CS104_Connection_sendASDU(CS104_Connection self, CS101_ASDU asdu)
 {
-    Frame frame = (Frame) T104Frame_create();
+    Frame frame = (Frame)T104Frame_create();
 
     CS101_ASDU_encode(asdu, frame);
 
@@ -1379,4 +1434,3 @@ CS104_Connection_isTransmitBufferFull(CS104_Connection self)
 {
     return isSentBufferFull(self);
 }
-
